#[macro_use] extern crate rocket;

use std::{fs::File, net::{IpAddr, Ipv4Addr}, sync::Mutex};

use fontloader::{writer::Visitor, Font, SvgWriter};
use rocket::{response::{content, status}, Config, State};

struct SharedFont {
	font: Mutex<Font>
}

#[get("/glyph/<glyph>")]
fn get_glyph(font: &State<SharedFont>, glyph: &str) -> status::Accepted<content::RawHtml<String>> {
	let Some(char) = glyph.chars().next() else {
		return status::Accepted(content::RawHtml("Please provide a char".into()));
	};

	let result = match font.font.lock().unwrap().get_data_for(char) {
		Ok(result) => result,
		Err(err) => return status::Accepted(content::RawHtml(err.to_string()))
	};

	let mut svg_buf = vec![];
	SvgWriter::new(&mut svg_buf).write(&result);

	status::Accepted(content::RawHtml(String::from_utf8(svg_buf).unwrap()))
}

#[launch]
fn rocket() -> _ {
	let _ = fontloader::init();

	let font = Font::new(File::open("/fonts/Helvetica.ttf").unwrap());
	let Ok(font) = font else {
        let err = font.unwrap_err().to_string();
        panic!("{err}");
	};

	let mut config = Config::release_default();
	config.address = IpAddr::V4(Ipv4Addr::new(0, 0, 0, 0));
	config.port = 8000;

	rocket::custom(config)
		.manage(SharedFont { font: Mutex::from(font) })
		.mount("/", routes![get_glyph])
}
